package com.up.mls.config;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Date;
import java.util.LinkedHashMap;
import java.util.Map;

import org.apache.commons.io.FilenameUtils;
import org.apache.commons.lang3.RandomStringUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.time.DateFormatUtils;
import org.codehaus.jackson.map.ObjectMapper;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.cache.annotation.CacheConfig;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Component;

import com.aliyun.oss.OSS;
import com.aliyun.oss.OSSClient;
import com.aliyun.oss.OSSClientBuilder;
import com.aliyun.oss.common.utils.BinaryUtil;
import com.aliyun.oss.common.utils.DateUtil;
import com.aliyun.oss.model.GetObjectRequest;
import com.aliyun.oss.model.MatchMode;
import com.aliyun.oss.model.OSSObject;
import com.aliyun.oss.model.ObjectMetadata;
import com.aliyun.oss.model.PolicyConditions;
import com.google.common.collect.Maps;

import lombok.Data;
import net.coobird.thumbnailator.Thumbnails;

@Data
@Component
@ConfigurationProperties(prefix = "aliyun-video.oss")
//@CacheConfig(cacheNames = "ossCache-video")
public class AliyunOssVideoConfig {
	private static final Logger log = LoggerFactory.getLogger(AliyunOssVideoConfig.class);

	private String bucket;
	private String endpoint;
	private String accessId;
	private String accessKey;
	private String dir;
	private String callBackUrl;
	private String callBackBody;
	private String callBackBodyType;

	// 过期时间1年
	public  final Long expireTime = 365 * 24 * 60 * 60L;
	
	@Cacheable(key = "#bucket+':'+#endpoint+':'+#accessId+':'+#accessKey+':'+#dir")
	public Map<String, String> getSig(String bucket, String endpoint, String accessId, String accessKey, String dir) {

		Map<String, String> respMap = null;
		OSS client = new OSSClientBuilder().build(endpoint, accessId, accessKey);
		try {
			long expireEndTime = System.currentTimeMillis() + expireTime * 1000;
			Date expiration = new Date(expireEndTime);
			PolicyConditions policyConds = new PolicyConditions();
			policyConds.addConditionItem(PolicyConditions.COND_CONTENT_LENGTH_RANGE, 0, 1048576000);
			policyConds.addConditionItem(MatchMode.StartWith, PolicyConditions.COND_KEY, dir);
			String postPolicy = client.generatePostPolicy(expiration, policyConds);
			byte[] binaryData = postPolicy.getBytes("utf-8");
			String encodedPolicy = BinaryUtil.toBase64String(binaryData);
			String postSignature = client.calculatePostSignature(postPolicy);
			respMap = new LinkedHashMap<String, String>();
			respMap.put("accessid", accessId);
			respMap.put("policy", encodedPolicy);
			respMap.put("signature", postSignature);
			respMap.put("expire", DateUtil.formatIso8601Date(expiration));
			respMap.put("dir", dir);
			respMap.put("host", getDefaultHost());
			respMap.put("expire", String.valueOf(expireEndTime / 1000));
			Map<String, String> callBackMap = Maps.newHashMap();
			callBackMap.put("callbackUrl", callBackUrl);
			callBackMap.put("callbackBody", callBackBody);
			callBackMap.put("callbackBodyType", callBackBodyType);
			respMap.put("callback",
					BinaryUtil.toBase64String(new ObjectMapper().writeValueAsString(callBackMap).getBytes()));
			log.info(new ObjectMapper().writeValueAsString(respMap));
			return respMap;

		} catch (Exception ex) {
			log.error("上传Aliyun Oss签名错误 ", ex);
		}
		client.shutdown();
		return respMap;

	}

	/**
	 * 裁剪
	 * 
	 * @param bucket
	 * @param endpoint
	 * @param accessId
	 * @param accessKey
	 * @param style
	 * @return
	 */
	public Map<String, String> getCrop(String bucket, String endpoint, String accessId, String accessKey, String key,
			String style) {
		Map<String, String> respMap = null;
		OSS client = new OSSClientBuilder().build(endpoint, accessId, accessKey);
		try {
			GetObjectRequest request = new GetObjectRequest(bucket, key);
			request.setProcess(style);
			client.getObject(request, new File("d:/example-resize.jpg"));
		} catch (Exception ex) {
			log.error("上传Aliyun Oss签名错误 ", ex);
		}
		client.shutdown();
		return respMap;
	}

	public static synchronized OSS  getOssClient(AliyunOssVideoConfig config) {
		OSS client = new OSSClientBuilder().build(config.getEndpoint(), config.getAccessId(), config.getAccessKey());
		return client;
	}

	public static String ossUploadUrl(String url, String  bucket,String key, OSS client,ObjectMetadata metadata)
			throws MalformedURLException, IOException {
		InputStream inputStream = new URL(url).openStream();
		client.putObject(bucket, key, inputStream,metadata);
		return AliyunOssVideoConfig.getHost(bucket, ((OSSClient) client).getEndpoint().getHost()) + "/" + key;
	}

	/**
	 * 
	 * @param client
	 * @param bucket
	 * @param key
	 * @param style
	 * @return
	 */
	public static String getCrop(OSS client, String bucket, String key,String newKey, String style) {
		try {
			GetObjectRequest request = new GetObjectRequest(bucket, key);
			request.setProcess(style);
			// client.getObject(request, new File(localKey));
			ObjectMetadata metadata = new ObjectMetadata();
			metadata.setContentType("image/jpeg");
			client.putObject(bucket, newKey, client.getObject(request).getObjectContent(), metadata);
		} catch (Exception ex) {
			log.error("AliyunOss裁剪图片错误 ", ex);
			return null;
		}
		return AliyunOssVideoConfig.getHost(bucket, ((OSSClient) client).getEndpoint().getHost()) + "/" + newKey;
	}

	@SuppressWarnings("unchecked")
	public Map<String, Integer> getInfo(String bucket, String endpoint, String accessId, String accessKey, String key,
			String style) {
		Map<String, Object> map = null;
		Map<String, Integer> resMap = Maps.newHashMap();
		OSS client = new OSSClientBuilder().build(endpoint, accessId, accessKey);
		try {
			GetObjectRequest request = new GetObjectRequest(bucket, key);
			request.setProcess(style);
			style = "image/info";
			request = new GetObjectRequest(bucket, key);
			request.setProcess(style);
			OSSObject ossObject = client.getObject(request);
			BufferedReader bf = new BufferedReader(new InputStreamReader(ossObject.getObjectContent(), "UTF-8"));
			// 最好在将字节流转换为字符流的时候 进行转码
			StringBuffer buffer = new StringBuffer();
			String line = "";
			while ((line = bf.readLine()) != null) {
				buffer.append(line);
			}
			map = JsonHelper.jsonToBean(buffer.toString(), Map.class);
			String width = ((Map<String, String>) map.get("ImageWidth")).get("value").toString();
			String height = ((Map<String, String>) map.get("ImageHeight")).get("value").toString();
			resMap.put("width", Integer.valueOf(width));
			resMap.put("height", Integer.valueOf(height));
		} catch (Exception ex) {
			log.error("上传Aliyun Oss签名错误 ", ex);
		}
		client.shutdown();
		return resMap;
	}

	@SuppressWarnings("unchecked")
	public Map<String, Integer> getInfo(OSS client, String bucket, String key) {
		Map<String, Object> map = null;
		Map<String, Integer> resMap = Maps.newHashMap();
		try {
			boolean found=client.doesObjectExist(bucket, key);
			if (found) {
				GetObjectRequest request = new GetObjectRequest(bucket, key);
				String style = "image/info";
				request.setProcess(style);
				OSSObject ossObject = client.getObject(request);
				BufferedReader bf = new BufferedReader(new InputStreamReader(ossObject.getObjectContent(), "UTF-8"));
				// 最好在将字节流转换为字符流的时候 进行转码
				StringBuffer buffer = new StringBuffer();
				String line = "";
				while ((line = bf.readLine()) != null) {
					buffer.append(line);
				}
				map = JsonHelper.jsonToBean(buffer.toString(), Map.class);
				String width = ((Map<String, String>) map.get("ImageWidth")).get("value").toString();
				String height = ((Map<String, String>) map.get("ImageHeight")).get("value").toString();
				resMap.put("width", Integer.valueOf(width));
				resMap.put("height", Integer.valueOf(height));
			}
		} catch (Exception ex) {
			log.error("获取图片信息错误 ", ex);
		}
		return resMap;
	}

	public static String getHost(String bucket, String endpoint) {
		return "http://" + bucket + "." + endpoint;
	}

	public String getDefaultHost() {
		return "http://" + this.getBucket() + "." + this.getEndpoint();
	}

	public static String getImgUrl(String host, String dir) {
		return host + dir;
	}

	public static String getOssKey(String dir) {
		return dir + "/" + getOssKey();
	}

	public static String getOssKey() {
		return RandomStringUtils.random(32, "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789");
	}

	/**
	 * aliyunoss图片缩略图生成
	 * 
	 * @param imgUrl
	 * @param bucket
	 * @param endpoint
	 * @param accessId
	 * @param accessKey
	 * @return
	 * @throws IOException
	 */
	public static String uploadOssUrlThump(String imgUrl, String bucket, String endpoint, String accessId,
			String accessKey) throws IOException {

		// 创建OSSClient实例
		OSS client = new OSSClientBuilder().build(endpoint, accessId, accessKey);
		StringBuilder imgsBuilder = new StringBuilder();
		URL url = new URL(imgUrl);
		// 上传网络流
		String dir = "upload/thumbnail/" + DateFormatUtils.format(new Date(), "yyyy-MM-dd");
		String host = getHost(bucket, endpoint);
		String ossKey = getOssKey();
		String suffix = FilenameUtils.getExtension(imgUrl);
		String osName = System.getProperty("os.name");
		File file = null;
		File directory = null;
		if (StringUtils.isNotBlank(osName) && osName.startsWith("Windows")) {
			directory = new File("d:\\template\\" + dir);
		} else {
			directory = new File("/opt/template/" + dir);
		}
		if (!directory.exists()) {
			directory.mkdirs();
		}
		file = new File(directory.getAbsolutePath() + "/" + ossKey + "." + suffix);
		Thumbnails.of(url).size(200, 240).toFile(file);
		InputStream inputStream = new FileInputStream(file);
		client.putObject(bucket, dir + "/" + ossKey, inputStream);
		// file.deleteOnExit();
		client.shutdown();
		return imgsBuilder.append(host).append("/").append(dir).append("/").append(ossKey).toString();
	}

	public static boolean delOssObject(String bucket, String endpoint, String accessId, String accessKey, String key) {
		OSS ossClient = new OSSClientBuilder().build(endpoint, accessId, accessKey);
		boolean result = false;
		try {
			if (ossClient.doesObjectExist(bucket, key)) {
				ossClient.deleteObject(bucket, key);
				result = true;
			}
		} catch (RuntimeException e) {
			e.printStackTrace();
		} finally {
			ossClient.shutdown();
		}
		return result;
	}
	
	
	public static boolean delOssObject(String bucket, OSS ossClient, String key) {
		boolean result = false;
		log.info("【删除OssObject】 bucket:{},key:{}",bucket,key);
		try {
			if (ossClient.doesObjectExist(bucket, key)) {
				//ossClient.deleteObject(bucket, key);
				//result = true;
			}
		} catch (RuntimeException e) {
			e.printStackTrace();
		} 
		return result;
	}

	public static void main(String[] args) throws IOException {

	}
}
