package com.up.mls.controller.system;

import java.util.List;
import java.util.Map;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.annotation.Secured;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;

import com.up.mls.annotation.Log;
import com.up.mls.common.constant.BusinessType;
import com.up.mls.controller.AbstractController;
import com.up.mls.service.system.IMenuService;
import com.up.mls.util.StringUtils;
import com.up.mls.vo.system.Menu;
import com.up.mls.vo.system.Role;
import com.up.mls.web.domain.AjaxResult;

/**
 * 菜单信息
 * 
 * @author j.x.j
 */
@Controller
@RequestMapping("/system/menu")
public class MenuController extends AbstractController {

	private String prefix = "system/menu";

	@Autowired
	private IMenuService menuService;

	@Secured("system:menu:view")
	@GetMapping()
	public String menu() {
		return prefix + "/menu";
	}

	@Secured("system:menu:list")
	@GetMapping("/list")
	@ResponseBody
	public List<Menu> list(Menu menu) {
		List<Menu> menuList = menuService.selectMenuList(menu);
		return menuList;
	}

	/**
	 * 删除菜单
	 */
	@Log(title = "菜单管理", action = BusinessType.DELETE)
	@Secured("system:menu:remove")
	@PostMapping("/remove/{menuId}")
	@ResponseBody
	public AjaxResult remove(@PathVariable("menuId") Long menuId) {
		if (menuService.selectCountMenuByParentId(menuId) > 0) {
			return error(1, "存在子菜单,不允许删除");
		}
		if (menuService.selectCountRoleMenuByMenuId(menuId) > 0) {
			return error(1, "菜单已分配,不允许删除");
		}
		return toAjax(menuService.deleteMenuById(menuId));
	}

	/**
	 * 新增
	 */
	@GetMapping("/add/{parentId}")
	public String add(@PathVariable("parentId") Long parentId, ModelMap mmap) {
		Menu menu = null;
		if (0L != parentId) {
			menu = menuService.selectMenuById(parentId);
		} else {
			menu = new Menu();
			menu.setMenuId(0L);
			menu.setMenuName("主目录");
		}
		mmap.put("menu", menu);
		return prefix + "/add";
	}

	/**
	 * 新增保存菜单
	 */
	@Log(title = "菜单管理", action = BusinessType.INSERT)
	@Secured("system:menu:add")
	@PostMapping("/add")
	@ResponseBody
	public AjaxResult addSave(Menu menu) {
		return toAjax(menuService.insertMenu(menu));
	}

	/**
	 * 修改菜单
	 */
	@GetMapping("/edit/{menuId}")
	public String edit(@PathVariable("menuId") Long menuId, ModelMap mmap) {
		mmap.put("menu", menuService.selectMenuById(menuId));
		return prefix + "/edit";
	}

	/**
	 * 修改保存菜单
	 */
	@Log(title = "菜单管理", action = BusinessType.UPDATE)
	@Secured("system:menu:edit")
	@PostMapping("/edit")
	@ResponseBody
	public AjaxResult editSave(Menu menu) {
		return toAjax(menuService.updateMenu(menu));
	}

	/**
	 * 选择菜单图标
	 */
	@GetMapping("/icon")
	public String icon() {
		return prefix + "/icon";
	}

	/**
	 * 校验菜单名称
	 */
	@PostMapping("/checkMenuNameUnique")
	@ResponseBody
	public String checkMenuNameUnique(Menu menu) {
		String uniqueFlag = "0";
		if (StringUtils.isNotNull(menu)) {
			uniqueFlag = menuService.checkMenuNameUnique(menu);
		}
		return uniqueFlag;
	}

	/**
	 * 加载角色菜单列表树
	 */
	@GetMapping("/roleMenuTreeData")
	@ResponseBody
	public List<Map<String, Object>> roleMenuTreeData(Role role) {
		List<Map<String, Object>> tree = menuService.roleMenuTreeData(role);
		return tree;
	}

	/**
	 * 加载所有菜单列表树
	 */
	@GetMapping("/menuTreeData")
	@ResponseBody
	public List<Map<String, Object>> menuTreeData(Role role) {
		List<Map<String, Object>> tree = menuService.menuTreeData();
		return tree;
	}

	/**
	 * 选择菜单树
	 */
	@GetMapping("/selectMenuTree/{menuId}")
	public String selectMenuTree(@PathVariable("menuId") Long menuId, ModelMap mmap) {
		mmap.put("menu", menuService.selectMenuById(menuId));
		return prefix + "/tree";
	}
}