/**
 * 
 */
package com.up.mls.interceptor;

import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Set;

import org.apache.commons.lang.StringUtils;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.reflect.MethodSignature;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.Ordered;
import org.springframework.dao.DataAccessException;
import org.springframework.data.redis.connection.RedisConnection;
import org.springframework.data.redis.core.RedisCallback;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.serializer.SerializationException;
import org.springframework.stereotype.Component;

import com.up.mls.annotation.CacheEvict;
import com.up.mls.annotation.Cacheable;

/**
 * @author mingguangz
 * 
 */
@Component
@Aspect
@SuppressWarnings({"unchecked","rawtypes"})
public class CacheAspect implements Ordered {
	private static final Logger logger = LoggerFactory.getLogger(CacheAspect.class);
	@Autowired
	private RedisTemplate redisTemplate;

	@Around("@annotation(com.up.mls.annotation.Cacheable)")
	public Object cacheable(ProceedingJoinPoint pjp) throws Throwable {
		Object result = null;
		try {
			MethodSignature joinPointObject = (MethodSignature) pjp.getSignature();  
			Method method = pjp.getTarget().getClass().getMethod(joinPointObject.getName(), joinPointObject.getParameterTypes());
			Cacheable cacheable = method.getAnnotation(Cacheable.class);
			String key = this.generateKey(cacheable.key(), pjp);
			result = this.get(key);
			if (result == null) {
				result = pjp.proceed();
				this.set(key,result,cacheable.expireTime());
				logger.debug(key+" Query from DB");
			} else {
				logger.debug(key+" Query from Redis");
			}
		} catch (Exception e) {
			result = pjp.proceed();
			logger.error(e.getMessage(), e);
		}
		return result;
	}
	
	@Around("@annotation(com.up.mls.annotation.CacheEvict)")
	public Object CacheEvict(ProceedingJoinPoint pjp) throws Throwable {
		try {
			MethodSignature joinPointObject = (MethodSignature) pjp.getSignature();  
			Method method = pjp.getTarget().getClass().getMethod(joinPointObject.getName(), joinPointObject.getParameterTypes());
			CacheEvict cacheEvict = method.getAnnotation(CacheEvict.class);
			List<String> keys = this.keys(this.generateKey(cacheEvict.key()+"*", null));
			this.del(keys);
		} catch (Exception e) {
			logger.error(e.getMessage(), e);
		}
		return pjp.proceed();
	}
	
	private String generateKey(String keyPrefix, ProceedingJoinPoint pjp) {
		StringBuffer key = new StringBuffer("");
		try {
			if(StringUtils.isNotEmpty(keyPrefix)) {
				key.append(keyPrefix).append("-");
			}
			if(pjp != null) {
				key.append(pjp.getTarget().getClass().getName()).append(".").append(pjp.getSignature().getName());
				Object [] parameters=pjp.getArgs();
				if(parameters!=null){
					for(int i=0;i<parameters.length;i++){
						if(i==0) key.append("(");
						key.append(parameters[i]==null?"":parameters[i].toString());
						if (i<parameters.length-1) key.append(",");
						else key.append(")");
					}
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		logger.debug("generate key: "+key);
		return key.toString();
	}
	
	private void set(final String key,final Object value,final long expireTime){
		this.redisTemplate.execute(new RedisCallback<Object>() {
			@Override
			public Object doInRedis(RedisConnection connection)
					throws DataAccessException {
				try {
					connection.setEx(redisTemplate.getStringSerializer().serialize(key), expireTime , redisTemplate.getValueSerializer().serialize(value));
				} catch (SerializationException e) {
					e.printStackTrace();
				}
				return null;
			}
		});
	}
	
	private Object get(final String key){
		return this.redisTemplate.execute(new RedisCallback<Object>() {
			@Override
			public Object doInRedis(RedisConnection connection)
					throws DataAccessException {
				try {
					return redisTemplate.getValueSerializer().deserialize(connection.get(redisTemplate.getStringSerializer().serialize(key)));
				} catch (SerializationException e) {
					e.printStackTrace();
				}
				return null;
			}
		});
	}
	
	private List<String> keys(final String pattern){
		return (List<String>) this.redisTemplate.execute(new RedisCallback<Object>() {
			@Override
			public Object doInRedis(RedisConnection connection)
					throws DataAccessException {
				Set<byte[]> keysB = connection.keys(redisTemplate.getStringSerializer().serialize(pattern));
				List<String> r = new ArrayList<String>();
				for(byte[] key : keysB){
					r.add(redisTemplate.getStringSerializer().deserialize(key).toString());
				}
				return r;
			}
		});
	}
	
	private Long del(final Collection<String> keys){
		return (Long) this.redisTemplate.execute(new RedisCallback<Long>() {
			@Override
			public Long doInRedis(RedisConnection connection)
					throws DataAccessException {
				long row = 0;
				for(String key : keys){
					row = row + connection.del(redisTemplate.getStringSerializer().serialize(key));
				}
				return row;
			}
		});
	}

	@Override
	public int getOrder() {
		return 2;
	}
}
