/**
 * 
 */
package com.up.mls.redis;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.dao.DataAccessException;
import org.springframework.data.redis.connection.RedisConnection;
import org.springframework.data.redis.core.RedisCallback;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.serializer.GenericToStringSerializer;
import org.springframework.data.redis.serializer.RedisSerializer;
import org.springframework.stereotype.Component;

/**
 * @author mingguangz
 *
 */
@SuppressWarnings({"rawtypes","unchecked"})
@Component
public class RedisBaseDaoBean implements Serializable, RedisBaseDao {

	private static final long serialVersionUID = 2419178268563094658L;
	@Autowired
	private RedisTemplate redisTemplate;
	
	/**
	 * key 返回给定key的value类型。返回 none 表示不存在key,string字符类型，list 链表类型 set 无序集合类型
	 */
	public String type(final String key){
		return (String) this.redisTemplate.execute(new RedisCallback<String>() {
			@Override
			public String doInRedis(RedisConnection connection)
					throws DataAccessException {
				return connection.type(redisTemplate.getStringSerializer().serialize(key)).code();
			}
		});
	}
	
	public boolean exists(final String key){
		return (Boolean) this.redisTemplate.execute(new RedisCallback<Boolean>() {
			@Override
			public Boolean doInRedis(RedisConnection connection)
					throws DataAccessException {
				return connection.exists(redisTemplate.getStringSerializer().serialize(key));
			}
		});
	}
	/**
	 * keys pattern 返回匹配指定模式的所有key
	 * keys t*
	 * keys t[ia]st
	 * keys t?st
	 */
	public List<String> keys(final String pattern){
		return (List<String>) this.redisTemplate.execute(new RedisCallback<Object>() {
			@Override
			public Object doInRedis(RedisConnection connection)
					throws DataAccessException {
				Set<byte[]> keysB = connection.keys(redisTemplate.getStringSerializer().serialize(pattern));
				List<String> r = new ArrayList<String>();
				for(byte[] key : keysB){
					r.add(redisTemplate.getStringSerializer().deserialize(key).toString());
				}
				return r;
			}
		});
	}
	/**
	 * 删除所有数据库中的所有key，此方法不会失败。慎用
	 */
	public void flushAll(){
		this.redisTemplate.execute(new RedisCallback<Object>() {
			@Override
			public Object doInRedis(RedisConnection connection)
					throws DataAccessException {
				connection.flushAll();
				return null;
			}
		});
	}
	/**
	 * expire key seconds 为key指定过期时间，单位是秒。返回1成功，0表示key已经设置过过期时间或者不存在
	 * @param key
	 * @param seconds
	 * @return
	 */
	public boolean expire(final String key, final long seconds){
		return (Boolean) this.redisTemplate.execute(new RedisCallback<Boolean>() {
			@Override
			public Boolean doInRedis(RedisConnection connection)
					throws DataAccessException {
				return connection.expire(redisTemplate.getStringSerializer().serialize(key), seconds);
			}
		});
	}
	/**
	 * 原子的重命名一个key,如果newkey存在返回失败，返回1表示成功，0失败。可能是oldkey不存在或者和newkey相同
	 */
	public boolean renameNX(final String oldName, final String newName){
		return (Boolean) this.redisTemplate.execute(new RedisCallback<Boolean>() {
			@Override
			public Boolean doInRedis(RedisConnection connection)
					throws DataAccessException {
				return connection.renameNX(redisTemplate.getStringSerializer().serialize(oldName), redisTemplate.getStringSerializer().serialize(newName));
			}
		});
	}
	
	public void set(final String key,final Serializable value){
		this.redisTemplate.execute(new RedisCallback<Object>() {
			@Override
			public Object doInRedis(RedisConnection connection)
					throws DataAccessException {
				connection.set(redisTemplate.getStringSerializer().serialize(key) , redisTemplate.getValueSerializer().serialize(value));
				return null;
			}
		});
	}
	
	public Object get(final String key){
		return this.redisTemplate.execute(new RedisCallback<Object>() {
			@Override
			public Object doInRedis(RedisConnection connection)
					throws DataAccessException {
				return redisTemplate.getValueSerializer().deserialize(connection.get(redisTemplate.getStringSerializer().serialize(key)));
			}
		});
	}
	
	public Long getIncr(final String key) {
		return (Long) this.redisTemplate.execute(new RedisCallback<Long>() {
			@Override
			public Long doInRedis(RedisConnection connection)
					throws DataAccessException {
				RedisSerializer<Long> redisSerializer = new GenericToStringSerializer<Long>(Long.class);
				return redisSerializer.deserialize(connection.get(redisTemplate.getStringSerializer().serialize(key)));
				/*redisTemplate.setValueSerializer(new GenericToStringSerializer<Long>(Long.class));
				return (Long) redisTemplate.getValueSerializer().deserialize(connection.get(redisTemplate.getStringSerializer().serialize(key)));*/
			}
		});
	}
	
	public Long incrBy(final String key,final long value){
		return (Long) this.redisTemplate.execute(new RedisCallback<Long>() {
			@Override
			public Long doInRedis(RedisConnection connection)
					throws DataAccessException {
				return connection.incrBy(redisTemplate.getStringSerializer().serialize(key),value);
			}
		});
	}
	
	public Long incr(final String key){
		return (Long) this.redisTemplate.execute(new RedisCallback<Long>() {
			@Override
			public Long doInRedis(RedisConnection connection)
					throws DataAccessException {
				return connection.incr(redisTemplate.getStringSerializer().serialize(key));
			}
		});
	}
	
	public void hMSet(final String key,final Map<String,Serializable> value){
		if(value==null || value.size()==0) return;
		this.redisTemplate.execute(new RedisCallback<Object>() {
			@Override
			public Object doInRedis(RedisConnection connection)
					throws DataAccessException {
				Map<byte[],byte[]> hashes = new HashMap<byte[], byte[]>();
				for(String k : value.keySet()){
					hashes.put(redisTemplate.getStringSerializer().serialize(k), redisTemplate.getValueSerializer().serialize(value.get(k)));
				}
				connection.hMSet(redisTemplate.getStringSerializer().serialize(key), hashes);
				return null;
			}
		});
	}
	
	public List<Object> hVals(final String key){
		return (List<Object>) this.redisTemplate.execute(new RedisCallback<Object>() {
			@Override
			public Object doInRedis(RedisConnection connection)
					throws DataAccessException {
				List<byte[]> vList =  connection.hVals(redisTemplate.getStringSerializer().serialize(key));
				List<Object> r = new ArrayList<Object>();
				for(byte[] b : vList){
					r.add(redisTemplate.getValueSerializer().deserialize(b));
				}
				return r;
			}
		});
	}
	
	public List<Object> hMGet(final String key, final String ...fields){
		return (List<Object>) this.redisTemplate.execute(new RedisCallback<Object>() {
			@Override
			public Object doInRedis(RedisConnection connection)
					throws DataAccessException {
				byte[] fb[] = new byte[fields.length][];
				for(int i=0;i<fields.length;i++){
					fb[i]=redisTemplate.getStringSerializer().serialize(fields[i]);
				}
				List<byte[]> vList =  connection.hMGet(redisTemplate.getStringSerializer().serialize(key), fb);
				List<Object> r = new ArrayList<Object>();
				for(byte[] b : vList){
					r.add(redisTemplate.getValueSerializer().deserialize(b));
				}
				return r;
			}
		});
	}
	
	public Map<String,Object> hMGetMap(final String key, final String ...fields){
		return (Map<String,Object>) this.redisTemplate.execute(new RedisCallback<Object>() {
			@Override
			public Object doInRedis(RedisConnection connection)
					throws DataAccessException {
				byte[] fb[] = new byte[fields.length][];
				for(int i=0;i<fields.length;i++){
					fb[i]=redisTemplate.getStringSerializer().serialize(fields[i]);
				}
				List<byte[]> vList =  connection.hMGet(redisTemplate.getStringSerializer().serialize(key), fb);
				Map<String,Object> r = new HashMap<String,Object>();
				for(int i=0;i<vList.size();i++){
					r.put(fields[i], redisTemplate.getValueSerializer().deserialize(vList.get(i)));
				}
				return r;
			}
		});
	}
	
	public Map<String,Object> hMGetMap(final String key){
		return (Map<String,Object>) this.redisTemplate.execute(new RedisCallback<Object>() {
			@Override
			public Object doInRedis(RedisConnection connection)
					throws DataAccessException {
				List<String> ks = hKeys(key);
				byte[] keys[] = new byte[ks.size()][];
				for(int i=0;i<ks.size();i++){
					keys[i]=redisTemplate.getStringSerializer().serialize(ks.get(i));
				}
				List<byte[]> vList =  connection.hMGet(redisTemplate.getStringSerializer().serialize(key), keys);
				Map<String,Object> r = new HashMap<String,Object>();
				for(int i=0;i<vList.size();i++){
					r.put(ks.get(i), redisTemplate.getValueSerializer().deserialize(vList.get(i)));
				}
				return r;
			}
		});
	}
	
	public Long hDel(final String key, final String ...fields){
		return (Long) this.redisTemplate.execute(new RedisCallback<Long>() {
			@Override
			public Long doInRedis(RedisConnection connection)
					throws DataAccessException {
				byte[] fb[] = new byte[fields.length][];
				for(int i=0;i<fields.length;i++){
					fb[i]=redisTemplate.getStringSerializer().serialize(fields[i]);
				}
				return connection.hDel(redisTemplate.getStringSerializer().serialize(key), fb);
			}
		});
	}
	
	public Long hLen(final String key){
		return (Long) this.redisTemplate.execute(new RedisCallback<Long>() {
			@Override
			public Long doInRedis(RedisConnection connection)
					throws DataAccessException {
				return connection.hLen(redisTemplate.getStringSerializer().serialize(key));
			}
		});
	}
	
	public List<String> hKeys(final String key){
		return (List<String>) this.redisTemplate.execute(new RedisCallback<Object>() {
			@Override
			public Object doInRedis(RedisConnection connection)
					throws DataAccessException {
				Set<byte[]> vList =  connection.hKeys(redisTemplate.getStringSerializer().serialize(key));
				List<String> r = new ArrayList<String>();
				for(byte[] b : vList){
					r.add(redisTemplate.getStringSerializer().deserialize(b).toString());
				}
				return r;
			}
		});
	}
	
	public Long del(final String ...keys){
		return (Long) this.redisTemplate.execute(new RedisCallback<Long>() {
			@Override
			public Long doInRedis(RedisConnection connection)
					throws DataAccessException {
				long row = 0;
				for(String key : keys){
					row = row + connection.del(redisTemplate.getStringSerializer().serialize(key));
				}
				return row;
			}
		});
	}
	
	public void lPush(final String key,final Serializable ...value){
		if(value==null || value.length==0) return;
		this.redisTemplate.execute(new RedisCallback<Object>() {
			@Override
			public Object doInRedis(RedisConnection connection)
					throws DataAccessException {
				for(Object v : value){
					connection.lPush(redisTemplate.getStringSerializer().serialize(key), redisTemplate.getValueSerializer().serialize(v));
				}
				return null;
			}
		});
	}
	
	public void lSet(final String key,final long index,final Serializable value){
		this.redisTemplate.execute(new RedisCallback<Object>() {
			@Override
			public Object doInRedis(RedisConnection connection)
					throws DataAccessException {
				connection.lSet(redisTemplate.getStringSerializer().serialize(key), index, redisTemplate.getValueSerializer().serialize(value));
				return null;
			}
		});
	}
	
	public List<Object> lRange(final String key,final long begin,final long end){
		return (List<Object>) this.redisTemplate.execute(new RedisCallback<Object>() {
			@Override
			public Object doInRedis(RedisConnection connection)
					throws DataAccessException {
				List<byte[]> vList =  connection.lRange(redisTemplate.getStringSerializer().serialize(key), begin, end);
				List<Object> r = new ArrayList<Object>();
				for(byte[] b : vList){
					r.add(redisTemplate.getValueSerializer().deserialize(b));
				}
				return r;
			}
		});
	}
	
	public Object lIndex(final String key,final long index){
		return this.redisTemplate.execute(new RedisCallback<Object>() {
			@Override
			public Object doInRedis(RedisConnection connection)
					throws DataAccessException {
				return redisTemplate.getValueSerializer().deserialize(connection.lIndex(redisTemplate.getStringSerializer().serialize(key), index));
			}
		});
	}
	
	public Long lRem(final String key,final long count,final Serializable value){
		return (Long) this.redisTemplate.execute(new RedisCallback<Long>() {
			@Override
			public Long doInRedis(RedisConnection connection)
					throws DataAccessException {
				return connection.lRem(redisTemplate.getStringSerializer().serialize(key), count, redisTemplate.getValueSerializer().serialize(value));
			}
		});
	}
	
	public Long lLen(final String key){
		return (Long) this.redisTemplate.execute(new RedisCallback<Long>() {
			@Override
			public Long doInRedis(RedisConnection connection)
					throws DataAccessException {
				return connection.lLen(redisTemplate.getStringSerializer().serialize(key));
			}
		});
	}
	/**
	 * 队列操作：LPUSH入队，RPOP出队
	 * @param key
	 * @return
	 */
	public Object rPop(final String key){
		return this.redisTemplate.execute(new RedisCallback<Object>() {
			@Override
			public Object doInRedis(RedisConnection connection)
					throws DataAccessException {
				return redisTemplate.getValueSerializer().deserialize(connection.rPop(redisTemplate.getStringSerializer().serialize(key)));
			}
		});
	}
	/**
	 * 堆栈操作：LPUSH压栈，LPOP出栈
	 * @param key
	 * @return
	 */
	public Object lPop(final String key){
		return this.redisTemplate.execute(new RedisCallback<Object>() {
			@Override
			public Object doInRedis(RedisConnection connection)
					throws DataAccessException {
				return redisTemplate.getValueSerializer().deserialize(connection.lPop(redisTemplate.getStringSerializer().serialize(key)));
			}
		});
	}
	
	public void sAdd(final String key,final Serializable ...value){
		if(value==null || value.length==0) return;
		this.redisTemplate.execute(new RedisCallback<Object>() {
			@Override
			public Object doInRedis(RedisConnection connection)
					throws DataAccessException {
				for(Object v : value){
					connection.sAdd(redisTemplate.getStringSerializer().serialize(key), redisTemplate.getValueSerializer().serialize(v));
				}
				return null;
			}
		});
	}
	
	public Long sRem(final String key,final Serializable value){
		return (Long) this.redisTemplate.execute(new RedisCallback<Long>() {
			@Override
			public Long doInRedis(RedisConnection connection)
					throws DataAccessException {
				return connection.sRem(redisTemplate.getStringSerializer().serialize(key), redisTemplate.getValueSerializer().serialize(value));
			}
		});
	}
	
	public Long sCard(final String key){
		return (Long) this.redisTemplate.execute(new RedisCallback<Long>() {
			@Override
			public Long doInRedis(RedisConnection connection)
					throws DataAccessException {
				return connection.sCard(redisTemplate.getStringSerializer().serialize(key));
			}
		});
	}
	
	public Set<Object> sMembers(final String key){
		return (Set<Object>) this.redisTemplate.execute(new RedisCallback<Object>() {
			@Override
			public Object doInRedis(RedisConnection connection)
					throws DataAccessException {
				Set<byte[]> vList =  connection.sMembers(redisTemplate.getStringSerializer().serialize(key));
				Set<Object> r = new HashSet<Object>();
				for(byte[] b : vList){
					r.add(redisTemplate.getValueSerializer().deserialize(b));
				}
				return r;
			}
		});
	}
	
	public Boolean sIsMember(final String key,final Serializable value){
		return (Boolean) this.redisTemplate.execute(new RedisCallback<Boolean>() {
			@Override
			public Boolean doInRedis(RedisConnection connection)
					throws DataAccessException {
				return connection.sIsMember(redisTemplate.getStringSerializer().serialize(key), redisTemplate.getValueSerializer().serialize(value));
			}
		});
	}
	
	public Object sPop(final String key){
		return this.redisTemplate.execute(new RedisCallback<Object>() {
			@Override
			public Object doInRedis(RedisConnection connection)
					throws DataAccessException {
				return redisTemplate.getValueSerializer().deserialize(connection.sPop(redisTemplate.getStringSerializer().serialize(key)));
			}
		});
	}
}
