package com.up.mls.service.system.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.google.common.base.Objects;
import com.up.mls.common.constant.SystemConstants;
import com.up.mls.common.constant.UserConstants;
import com.up.mls.mapper.system.DeptMapper;
import com.up.mls.service.system.IDeptService;
import com.up.mls.util.SpringSecurityUtils;
import com.up.mls.util.StringUtils;
import com.up.mls.vo.system.Dept;

/**
 * 部门管理 服务实现
 * 
 * @author j.x.j
 */
@Service
public class DeptServiceImpl implements IDeptService {
	@Autowired
	private DeptMapper deptMapper;

	/**
	 * 查询部门管理数据
	 * 
	 * @return 部门信息集合
	 */
	@Override
	public List<Dept> selectDeptList(Dept dept) {
		return deptMapper.selectDeptList(dept);
	}

	/**
	 * 查询部门所有数据
	 * 
	 * @return 部门信息集合
	 */
	@Override
	public List<Dept> selectDeptAll() {
		return deptMapper.selectDeptAll();
	}

	/**
	 * 查询部门管理树
	 * 
	 * @return 所有部门信息
	 */
	@Override
	public List<Map<String, Object>> selectDeptTree() {
		List<Map<String, Object>> trees = new ArrayList<Map<String, Object>>();
		List<Dept> deptList = deptMapper.selectDeptAll();

		for (Dept dept : deptList) {
			if (Objects.equal(SystemConstants.NORMAL, dept.getStatus())) {
				Map<String, Object> deptMap = new HashMap<String, Object>();
				deptMap.put("id", dept.getDeptId());
				deptMap.put("pId", dept.getParentId());
				deptMap.put("name", dept.getDeptName());
				deptMap.put("title", dept.getDeptName());
				trees.add(deptMap);
			}
		}
		return trees;
	}

	/**
	 * 查询部门人数
	 * 
	 * @param parentId
	 *            部门ID
	 * @return 结果
	 */
	@Override
	public int selectDeptCount(Long parentId) {
		Dept dept = new Dept();
		dept.setParentId(parentId);
		return deptMapper.selectDeptCount(dept);
	}

	/**
	 * 查询部门是否存在用户
	 * 
	 * @param deptId
	 *            部门ID
	 * @return 结果 true 存在 false 不存在
	 */
	@Override
	public boolean checkDeptExistUser(Long deptId) {
		int result = deptMapper.checkDeptExistUser(deptId);
		return result > 0 ? true : false;
	}

	/**
	 * 删除部门管理信息
	 * 
	 * @param deptId
	 *            部门ID
	 * @return 结果
	 */
	@Override
	public int deleteDeptById(Long deptId) {
		return deptMapper.deleteDeptById(deptId);
	}

	/**
	 * 新增保存部门信息
	 * 
	 * @param dept
	 *            部门信息
	 * @return 结果
	 */
	@Override
	public int insertDept(Dept dept) {
		Dept info = deptMapper.selectDeptById(dept.getParentId());
		dept.setCreateBy(SpringSecurityUtils.getUserId());
		dept.setAncestors(info.getAncestors() + "," + dept.getParentId());
		return deptMapper.insertDept(dept);
	}

	/**
	 * 修改保存部门信息
	 * 
	 * @param dept
	 *            部门信息
	 * @return 结果
	 */
	@Override
	public int updateDept(Dept dept) {
		Dept info = deptMapper.selectDeptById(dept.getParentId());
		String ancestors = info.getAncestors() + "," + dept.getParentId();
		dept.setUpdateBy(SpringSecurityUtils.getUserId());
		dept.setAncestors(ancestors);
		updateDeptChildren(dept.getDeptId(), ancestors);
		return deptMapper.updateDept(dept);
	}

	/**
	 * 修改子元素关系
	 * 
	 * @param deptId
	 *            部门ID
	 * @param ancestors
	 *            元素列表
	 */
	public void updateDeptChildren(Long deptId, String ancestors) {
		Dept dept = new Dept();
		dept.setParentId(deptId);
		List<Dept> childrens = deptMapper.selectDeptList(dept);
		for (Dept children : childrens) {
			children.setAncestors(ancestors + "," + dept.getParentId());
		}
		if (childrens.size() > 0) {
			deptMapper.updateDeptChildren(childrens);
		}
	}

	/**
	 * 根据部门ID查询信息
	 * 
	 * @param deptId
	 *            部门ID
	 * @return 部门信息
	 */
	@Override
	public Dept selectDeptById(Long deptId) {
		return deptMapper.selectDeptById(deptId);
	}

	/**
	 * 校验部门名称是否唯一
	 * 
	 * @param dept
	 *            部门信息
	 * @return 结果
	 */
	@Override
	public String checkDeptNameUnique(Dept dept) {
		Long deptId = StringUtils.isNull(dept.getDeptId()) ? -1L : dept.getDeptId();
		Dept info = deptMapper.checkDeptNameUnique(dept.getDeptName());
		if (StringUtils.isNotNull(info) && info.getDeptId().longValue() != deptId.longValue()) {
			return UserConstants.DEPT_NAME_NOT_UNIQUE;
		}
		return UserConstants.DEPT_NAME_UNIQUE;
	}

	public static void main(String[] args) {
		String a = null, b = null;
		System.out.println(Objects.equal(a, b));
		System.out.println(java.util.Objects.equals(a, b));
	}
}
