package com.up.mls.service.system.impl;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.stereotype.Service;

import com.up.mls.common.constant.UserConstants;
import com.up.mls.mapper.system.MenuMapper;
import com.up.mls.mapper.system.PostMapper;
import com.up.mls.mapper.system.RoleMapper;
import com.up.mls.mapper.system.UserMapper;
import com.up.mls.mapper.system.UserRoleMapper;
import com.up.mls.service.system.IUserService;
import com.up.mls.support.Convert;
import com.up.mls.util.SpringSecurityUtils;
import com.up.mls.util.StringUtils;
import com.up.mls.vo.system.Post;
import com.up.mls.vo.system.Role;
import com.up.mls.vo.system.User;
import com.up.mls.vo.system.UserRole;

/**
 * 用户 业务层处理
 * 
 * @author j.x.j
 */
@Service
public class UserServiceImpl implements IUserService, UserDetailsService {
	@Autowired
	private UserMapper userMapper;

	@Autowired
	private RoleMapper roleMapper;

	@Autowired
	private PostMapper postMapper;


	@Autowired
	private UserRoleMapper userRoleMapper;

	@Autowired
	private MenuMapper menuMapper;

	/**
	 * 根据条件分页查询用户对象
	 * 
	 * @param user
	 *            用户信息
	 * 
	 * @return 用户信息集合信息
	 */
	@Override
	public List<User> selectUserList(User user) {
//		if(user.getrole) {
//			
//		}
		return userMapper.selectUserList(user);
	}

	/**
	 * 通过用户名查询用户
	 * 
	 * @param userName
	 *            用户名
	 * @return 用户对象信息
	 */
	@Override
	public User selectUserByLoginName(String userName) {
		return userMapper.selectUserByLoginName(userName);
	}

	/**
	 * 通过手机号码查询用户
	 * 
	 * @param userName
	 *            用户名
	 * @return 用户对象信息
	 */
	@Override
	public User selectUserByPhoneNumber(String phoneNumber) {
		return userMapper.selectUserByPhoneNumber(phoneNumber);
	}

	/**
	 * 通过邮箱查询用户
	 * 
	 * @param email
	 *            邮箱
	 * @return 用户对象信息
	 */
	@Override
	public User selectUserByEmail(String email) {
		return userMapper.selectUserByEmail(email);
	}

	/**
	 * 通过用户ID查询用户
	 * 
	 * @param userId
	 *            用户ID
	 * @return 用户对象信息
	 */
	@Override
	public User selectUserById(Long userId) {
		return userMapper.selectUserById(userId);
	}

	/**
	 * 通过用户ID删除用户
	 * 
	 * @param userId
	 *            用户ID
	 * @return 结果
	 */
	@Override
	public int deleteUserById(Long userId) {
		// 删除用户与角色关联
		userRoleMapper.deleteUserRoleByUserId(userId);
		return userMapper.deleteUserById(userId);
	}

	/**
	 * 批量删除用户信息
	 * 
	 * @param ids
	 *            需要删除的数据ID
	 * @return 结果
	 */
	@Override
	public int deleteUserByIds(String ids) throws Exception {
		Long[] userIds = Convert.toLongArray(ids);
		for (Long userId : userIds) {
			if (User.isAdmin(userId)) {
				throw new Exception("不允许删除超级管理员用户");
			}
		}
		return userMapper.deleteUserByIds(userIds);
	}

	/**
	 * 新增保存用户信息
	 * 
	 * @param user
	 *            用户信息
	 * @return 结果
	 */
	@Override
	public int insertUser(User user) {
		// user.setPassword(passwordEncoder.encode(user.getPassword()));
		BCryptPasswordEncoder b = new BCryptPasswordEncoder();
		user.setPassword(b.encode(user.getPassword()));
		user.setCreateBy(SpringSecurityUtils.getUserId());
		// 新增用户信息
		int rows = userMapper.insertUser(user);
		// 新增用户岗位关联
		//insertUserPost(user);
		// 新增用户与角色管理
		insertUserRole(user);
		return rows;
	}

	/**
	 * 修改保存用户信息
	 * 
	 * @param user
	 *            用户信息
	 * @return 结果
	 */
	@Override
	public int updateUser(User user) {
		Long userId = user.getUserId();
		user.setUpdateBy(SpringSecurityUtils.getUserId());
		// 删除用户与角色关联
		userRoleMapper.deleteUserRoleByUserId(userId);
		// 新增用户与角色管理
		insertUserRole(user);
		// 删除用户与岗位关联
		//userPostMapper.deleteUserPostByUserId(userId);
		// 新增用户与岗位管理
		//insertUserPost(user);
		return userMapper.updateUser(user);
	}
	
	@Override
	public int updateUserStatus(User user) {
		user.setUpdateBy(SpringSecurityUtils.getUserId());
		user.setUpdateTime(new Date());
		return userMapper.updateUser(user);
	}

	/**
	 * 修改用户个人详细信息
	 * 
	 * @param user
	 *            用户信息
	 * @return 结果
	 */
	@Override
	public int updateUserInfo(User user) {
		return userMapper.updateUser(user);
	}

	/**
	 * 修改用户密码
	 * 
	 * @param user
	 *            用户信息
	 * @return 结果
	 */
	@Override
	public int resetUserPwd(User user) {
		// user.setPassword(passwordEncoder.encode(user.getPassword()));
		BCryptPasswordEncoder b = new BCryptPasswordEncoder();
		user.setPassword(b.encode(user.getPassword()));
		return updateUserInfo(user);
	}

	/**
	 * 新增用户角色信息
	 * 
	 * @param user
	 *            用户对象
	 */
	public void insertUserRole(User user) {
		// 新增用户与角色管理
		List<UserRole> list = new ArrayList<UserRole>();
		if(Objects.nonNull(user.getRoleIds())) {
			for (Long roleId : user.getRoleIds()) {
				UserRole ur = new UserRole();
				ur.setUserId(user.getUserId());
				ur.setRoleId(roleId);
				list.add(ur);
			}
			if (list.size() > 0) {
				userRoleMapper.batchUserRole(list);
			}
		}
	}


	/**
	 * 校验用户名称是否唯一
	 * 
	 * @param loginName
	 *            用户名
	 * @return
	 */
	@Override
	public String checkLoginNameUnique(String loginName) {
		int count = userMapper.checkLoginNameUnique(loginName);
		if (count > 0) {
			return UserConstants.USER_NAME_NOT_UNIQUE;
		}
		return UserConstants.USER_NAME_UNIQUE;
	}

	/**
	 * 校验用户名称是否唯一
	 *
	 * @param phonenumber
	 *            用户名
	 * @return
	 */
	@Override
	public String checkPhoneUnique(User user) {
		Long userId = StringUtils.isNull(user.getUserId()) ? -1L : user.getUserId();
		User info = userMapper.checkPhoneUnique(user.getPhonenumber());
		if (StringUtils.isNotNull(info) && info.getUserId().longValue() != userId.longValue()) {
			return UserConstants.USER_PHONE_NOT_UNIQUE;
		}
		return UserConstants.USER_PHONE_UNIQUE;
	}

	/**
	 * 校验email是否唯一
	 *
	 * @param email
	 *            用户名
	 * @return
	 */
	@Override
	public String checkEmailUnique(User user) {
		Long userId = StringUtils.isNull(user.getUserId()) ? -1L : user.getUserId();
		User info = userMapper.checkEmailUnique(user.getEmail());
		if (StringUtils.isNotNull(info) && info.getUserId().longValue() != userId.longValue()) {
			return UserConstants.USER_EMAIL_NOT_UNIQUE;
		}
		return UserConstants.USER_EMAIL_UNIQUE;
	}

	/**
	 * 查询用户所属角色组
	 * 
	 * @param userId
	 *            用户ID
	 * @return 结果
	 */
	@Override
	public String selectUserRoleGroup(Long userId) {
		List<Role> list = roleMapper.selectRolesByUserId(userId);
		StringBuffer idsStr = new StringBuffer();
		for (Role role : list) {
			idsStr.append(role.getRoleName()).append(",");
		}
		if (StringUtils.isNotEmpty(idsStr.toString())) {
			return idsStr.substring(0, idsStr.length() - 1);
		}
		return idsStr.toString();
	}

	/**
	 * 查询用户所属岗位组
	 * 
	 * @param userId
	 *            用户ID
	 * @return 结果
	 */
	@Override
	public String selectUserPostGroup(Long userId) {
		List<Post> list = postMapper.selectPostsByUserId(userId);
		StringBuffer idsStr = new StringBuffer();
		for (Post post : list) {
			idsStr.append(post.getPostName()).append(",");
		}
		if (StringUtils.isNotEmpty(idsStr.toString())) {
			return idsStr.substring(0, idsStr.length() - 1);
		}
		return idsStr.toString();
	}

	@Override
	public UserDetails loadUserByUsername(String loginName) {
		System.out.println("loginName : "+loginName);
		User user = userMapper.selectUserByLoginName(loginName);
		List<SimpleGrantedAuthority> auths = menuMapper.selectMenusPermByUserId(user.getUserId()).stream().filter(m->StringUtils.isNotBlank(m.getPerms()))
				.map(m -> new SimpleGrantedAuthority(m.getPerms())).collect(Collectors.toList());
		user.setAuthorities(auths);
		auths.forEach(System.out::println);
		System.out.println("user : "+user);
		user.setPassword("{bcrypt}" + user.getPassword());
		return user;
	}

	@Override
	public List<User> selectUserByRole(String roleId, String roleName, String companyId) {
		// TODO Auto-generated method stub
		return userMapper.selectUserByRole(roleId, roleName, companyId);
	}

}
